@@ -7,7 +7,7 @@
 """
 from __future__ import division
 
-from math import sqrt, pi, sin, cos, atan2
+from math import sqrt, pi, sin, cos, atan2, degrees, radians
 import warnings
 
 from numpy import mean
@@ -93,7 +93,7 @@ class Station(object):
     _detectors = None
 
     def __init__(self, cluster, station_id, position, angle, detectors=None,
-                 station_number=None):
+                 number=None):
         """Initialize station
 
         :param cluster: cluster this station is a part of
@@ -106,17 +106,17 @@ class Station(object):
             Orientation is either 'UD' or 'LR' meaning an up-down or
             left-right orientation of the long side of the detector
             respectively.
-        :param station_number: optional unique identifier for a station
-            this can be used by the cluster to find a specific station
-            and makes it easier to link to a real station. If not given
-            it will be equal to the station_id.
+        :param number: optional unique identifier for a station this can
+            be used by the cluster to find a specific station and makes
+            it easier to link to a real station. If not given it will be
+            equal to the station_id.
 
         """
         self.cluster = cluster
         self.station_id = station_id
         self.position = position
         self.angle = angle
-        self.station_number = station_number if station_number else station_id
+        self.number = number if number else station_id
 
         if detectors is None:
             # detector positions for a standard station
@@ -209,8 +209,7 @@ class BaseCluster(object):
         self._x, self._y = position
         self._alpha = angle
 
-    def _add_station(self, position, angle, detectors=None,
-                     station_number=None):
+    def _add_station(self, position, angle, detectors=None, number=None):
         """Add a station to the cluster
 
         :param position: tuple of (x, y) values
@@ -221,16 +220,17 @@ class BaseCluster(object):
             Orientation is either 'UD' or 'LR' meaning an up-down or
             left-right orientation of the long side of the detector
             respectively.
-        :param station_number: optional unique identifier for a station
-            this can later be used to find a specific station and makes
-            it easier to link to a real station. If not given it will be
-            equal to the station_id generated by this function. Either
-            use this for all added stations or for none.
+        :param number: optional unique identifier for a station this can
+            later be used to find a specific station and makes it easier
+            to link to a real station. If not given it will be equal to
+            the station_id generated by this function. Either use this
+            for all added stations or for none.
 
         Example::
 
             >>> cluster = BaseCluster()
-            >>> cluster._add_station((0, 0), pi / 2, [(-5, 0, 'UD'), (5, 0, 'UD')])
+            >>> cluster._add_station((0, 0), pi / 2,
+            ...                      [(-5, 0, 'UD'), (5, 0, 'UD')])
 
         """
         # Need to make _stations an instance variable to be able to
@@ -240,17 +240,17 @@ class BaseCluster(object):
 
         station_id = len(self._stations)
         self._stations.append(Station(self, station_id, position, angle,
-                                      detectors, station_number))
+                                      detectors, number))
 
     @property
     def stations(self):
         return self._stations
 
-    def get_station(self, station_number):
+    def get_station(self, number):
         """Get a station by its number"""
 
         for station in self._stations:
-            if station_number == station.station_number:
+            if number == station.number:
                 return station
 
     def get_xyalpha_coordinates(self):
@@ -298,8 +298,13 @@ class BaseCluster(object):
         self._alpha = alpha
 
     def calc_r_and_phi_for_stations(self, s1, s2):
-        """Calculate angle between detectors (phi1, phi2)"""
+        """Calculate distance between and direction of two stations
 
+        :param s1, s2: The station ids for the two stations.
+        :return: r, phi; the distance between and direction of the two
+            given stations.
+
+        """
         x1, y1, alpha1 = self.stations[s1].get_xyalpha_coordinates()
         x2, y2, alpha2 = self.stations[s2].get_xyalpha_coordinates()
 
@@ -325,6 +330,46 @@ class BaseCluster(object):
         return x0, y0
 
 
+class RAlphaBetaStations(BaseCluster):
+
+    """Add detectors to stations using r, alpha coordinates
+
+    This is meant for data from the Publicdb Database API, which
+    uses that coordinate system.
+
+    """
+
+    def _add_station(self, position, detectors, number=None):
+        """Add a station to the cluster
+
+        :param position: tuple of (x, y) values
+        :param detectors: list of tuples.  Each tuple consists of (r, alpha,
+            beta) these define the position of the detector relative to
+            the GPS and the orientation of the detector. r is the
+            distance between the center of the scintillator and the GPS
+            in meters. alpha is the clock-wise turning angle between
+            North and the detector relative to the GPS in degrees. beta
+            is, like alpha, the clock-wise turning angle between North
+            and the long side of the detector.
+        :param number: optional unique identifier for a station this can
+            later be used to find a specific station and makes it easier
+            to link to a real station. If not given it will be equal to
+            the station_id generated by this function. Either use this
+            for all added stations or for none.
+
+        Example::
+
+            >>> cluster = RAlphaBetaStations()
+            >>> cluster._add_station((0, 0), [(5, 90, 0), (5, 270, 0)], 104)
+
+        """
+        detectors = [(sin(radians(alpha)) * r, cos(radians(alpha)) * r, 'UD')
+                     for r, alpha, _ in detectors]
+
+        super(RAlphaBetaStations, self)._add_station(position, 0, detectors,
+                                                     number)
+
+
 class SimpleCluster(BaseCluster):
     """Define a simple cluster containing four stations"""
 
@@ -472,3 +517,76 @@ class ScienceParkCluster(BaseCluster):
                 raise RuntimeError("Programming error. Station unknown.")
 
             self._add_station((easting, northing), alpha, detectors, station)
+
+
+class HiSPARCStations(RAlphaBetaStations):
+
+    """A cluster containing any real station from the HiSPARC network
+
+    The gps position and number of detectors are taken from the API.
+    The detector positions are retrieved if available, otherwise
+    default values are used!
+
+    :param stations: A list of station numbers to include. The
+        coordinates are retrieved from the Public Database API.
+        The first station is placed at the origin of the cluster.
+
+    Example::
+
+        >>> cluster = HiSPARCStations([7001, 7002, 7003])
+
+    """
+
+    def __init__(self, stations):
+        super(HiSPARCStations, self).__init__()
+
+        for i, station in enumerate(stations):
+            try:
+                station_info = sapphire.api.Station(station)
+            except:
+                warnings.warn('Could not get info for station %d, Skipping.' %
+                              station, UserWarning)
+            location = station_info.location()
+            n_detectors = station_info.n_detectors
+            detectors = station_info.detectors()
+            gps = (location['latitude'],
+                   location['longitude'],
+                   location['altitude'])
+
+            if i == 0:
+                transformation = \
+                    transformations.FromWGS84ToENUTransformation(gps)
+
+            easting, northing, up = transformation.transform(gps)
+            alpha = 0
+
+            # Default positions in (r, alpha, beta)
+            if n_detectors == 2:
+                default_detectors = [(5, 90, 0), (5, 270, 0)]
+            elif n_detectors == 4:
+                default_detectors = [(8.66, 0, 0), (2.89, 0, 0),
+                                     (5, -90, 90), (5, 90, 90)]
+            else:
+                raise RuntimeError("Detector count unknown for station %d." %
+                                   station)
+            detectors = [
+                (d['radius'] if d['radius'] else default_detectors[i][0],
+                 d['alpha'] if d['alpha'] else default_detectors[i][1],
+                 d['beta'] if d['beta'] else default_detectors[i][2])
+                for i, d in enumerate(detectors)]
+
+            self._add_station((easting, northing), detectors, station)
+
+        warnings.warn('Detector positions may be wrong, defaults are used '
+                      'when not available from API!', UserWarning)
+
+
+class HiSPARCNetwork(HiSPARCStations):
+
+    """A cluster containing all station from the HiSPARC network"""
+
+    def __init__(self):
+        network = sapphire.api.Network()
+        stations = [station['number'] for station in network.stations()]
+        super(HiSPARCNetwork, self).__init__(stations)
+
