@@ -43,8 +43,10 @@ def download_data(file, group, station_id, start, end, get_blobs=False):
         >>> import tables
         >>> import datetime
         >>> import sapphire.publicdb
-        >>> data = tables.openFile('data.h5', 'w')
-        >>> sapphire.publicdb.download_data(data, '/s501', 501, datetime.datetime(2010, 9, 1), datetime.datetime(2010, 9, 2))
+        >>> data = tables.open_file('data.h5', 'w')
+        >>> start = datetime.datetime(2010, 9, 1)
+        >>> end = datetime.datetime(2010, 9, 2)
+        >>> sapphire.publicdb.download_data(data, '/s501', 501, start, end)
         INFO:hisparc.publicdb:2010-09-01 00:00:00 None
         INFO:hisparc.publicdb:Getting server data URL (2010-09-01 00:00:00)
         INFO:hisparc.publicdb:Downloading data...
@@ -82,16 +84,16 @@ def _store_data(dst_file, dst_group, src_filename, t0, t1):
 
     """
     # Open in rw mode, need to update blob idxs, if necessary
-    with tables.openFile(src_filename, 'a') as src_file:
-        src_group = src_file.listNodes('/')[0]
+    with tables.open_file(src_filename, 'a') as src_file:
+        src_group = src_file.list_nodes('/')[0]
         dst_group = _get_or_create_group(dst_file, dst_group)
 
         if 'blobs' in dst_group:
-            len_blobs = len(dst_file.getNode(dst_group, 'blobs'))
+            len_blobs = len(dst_file.get_node(dst_group, 'blobs'))
         else:
             len_blobs = 0
 
-        for node in src_file.listNodes(src_group):
+        for node in src_file.list_nodes(src_group):
             dst_node = _get_or_create_node(dst_file, dst_group, node)
 
             if node.name == 'blobs':
@@ -108,11 +110,11 @@ def _store_data(dst_file, dst_group, src_filename, t0, t1):
                          calendar.timegm(t1.utctimetuple()))
 
                 if len_blobs:
-                    for row in node.readWhere(cond):
+                    for row in node.read_where(cond):
                         row['traces'] += len_blobs
                         dst_node.append([tuple(row)])
                 else:
-                    for row in node.readWhere(cond):
+                    for row in node.read_where(cond):
                         dst_node.append([tuple(row)])
 
             elif node.name == 'errors' and len_blobs:
@@ -153,7 +155,8 @@ def datetimerange(start, stop):
         ...                        datetime.datetime(2010, 1, 1, 13)):
         ...     x
         ...
-        (datetime.datetime(2010, 1, 1, 11, 0), datetime.datetime(2010, 1, 1, 13, 0))
+        (datetime.datetime(2010, 1, 1, 11, 0),
+         datetime.datetime(2010, 1, 1, 13, 0))
 
         >>> for x in datetimerange(datetime.datetime(2010, 1, 1, 11),
         ...                        datetime.datetime(2010, 1, 2)):
@@ -166,7 +169,8 @@ def datetimerange(start, stop):
         ...     x
         ...
         (datetime.datetime(2010, 1, 1, 11, 0), None)
-        (datetime.datetime(2010, 1, 2, 0, 0), datetime.datetime(2010, 1, 2, 13, 0))
+        (datetime.datetime(2010, 1, 2, 0, 0),
+         datetime.datetime(2010, 1, 2, 13, 0))
 
         >>> for x in datetimerange(datetime.datetime(2010, 1, 1, 11),
         ...                        datetime.datetime(2010, 1, 5, 13)):
@@ -176,7 +180,8 @@ def datetimerange(start, stop):
         (datetime.datetime(2010, 1, 2, 0, 0), None)
         (datetime.datetime(2010, 1, 3, 0, 0), None)
         (datetime.datetime(2010, 1, 4, 0, 0), None)
-        (datetime.datetime(2010, 1, 5, 0, 0), datetime.datetime(2010, 1, 5, 13, 0))
+        (datetime.datetime(2010, 1, 5, 0, 0),
+         datetime.datetime(2010, 1, 5, 13, 0))
 
     """
     if start.date() == stop.date():
@@ -184,8 +189,8 @@ def datetimerange(start, stop):
         return
     else:
         yield start, None
-        cur = start.replace(hour=0, minute=0, second=0, microsecond=0) + \
-              datetime.timedelta(days=1)
+        cur = (start.replace(hour=0, minute=0, second=0, microsecond=0) +
+               datetime.timedelta(days=1))
         while cur.date() < stop.date():
             yield cur, None
             cur += datetime.timedelta(days=1)
@@ -199,10 +204,10 @@ def _get_or_create_group(file, group):
     """Get or create a group in the datafile"""
 
     try:
-        group = file.getNode(group)
+        group = file.get_node(group)
     except tables.NoSuchNodeError:
         parent, newgroup = os.path.split(group)
-        file.createGroup(parent, newgroup, 'Data group',
+        file.create_group(parent, newgroup, 'Data group',
                          createparents=True)
     return group
 
@@ -211,13 +216,13 @@ def _get_or_create_node(file, group, src_node):
     """Get or create a node based on a source node"""
 
     try:
-        node = file.getNode(group, src_node.name)
+        node = file.get_node(group, src_node.name)
     except tables.NoSuchNodeError:
         if type(src_node) == tables.Table:
-            node = file.createTable(group, src_node.name,
+            node = file.create_table(group, src_node.name,
                                     src_node.description, src_node.title)
         elif type(src_node) == tables.VLArray:
-            node = file.createVLArray(group, src_node.name, src_node.atom,
+            node = file.create_vlarray(group, src_node.name, src_node.atom,
                                       src_node.title)
         else:
             raise Exception("Unknown node class: %s" % type(src_node))
